import 'package:flutter/material.dart';
import 'package:flutter_web_browser/flutter_web_browser.dart';
import 'package:flutter_app_auth_wrapper/flutter_app_auth_wrapper.dart';
import 'package:url_launcher/url_launcher.dart';
import 'dart:developer';
import 'dart:convert';

import '../../config.dart';

class Home extends StatefulWidget {
  Home({@required this.mode, @required this.invokerURL, @required this.setMode, @required this.host});
  final String mode;
  final String invokerURL;
  final String host;
  final void Function(String) setMode;

  @override
  _HomeState createState() => _HomeState();
}

const hosts = [
  'https://demo1.vereign.com',
  'https://demo2.vereign.com',
  'https://rosengeorgiev.dev.vereign.com',
  'https://borisdimitrov.dev.vereign.com',
  'https://integration.vereign.com',
  'https://staging.vereign.com',
  'https://damyanmitev.dev.vereign.com',
  'https://gospodinbodurov.dev.vereign.com',
  'https://alexeylunin.dev.vereign.com',
  'https://igormarkin.dev.vereign.com',
  'https://mikhaildorokhovich.dev.vereign.com',
  'https://sashailieva.dev.vereign.com',
  'https://alexanderholodov.dev.vereign.com',
  'https://olguncengiz.dev.vereign.com',
  'https://viktorpopov.dev.vereign.com',
  'https://kalincanov.dev.vereign.com',
  'https://alexeykuklin.dev.vereign.com'
];

class _HomeState extends State<Home> {
  String _host = Config.appFlavor == Flavor.DEVELOPMENT ? hosts[0] : Config.DEFAULT_APP_HOST;

  @override
  initState() {
    super.initState();
    showMode(widget.mode);

    FlutterAppAuthWrapper.eventStream().listen((data) {
      var token = json.decode(data.toString())["access_token"];
      _showAlert(token);
    });
  }

  Future<void> _showAlert(token) {
    return showDialog<void>(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          title: Text('Authorization success'),
          actions: <Widget>[
            FlatButton(
              child: Text('Go back'),
              onPressed: () async {
                Navigator.of(context).pop();
                widget.setMode("");

                log("Open ${widget.invokerURL}");
                try {
                  await launch("${widget.invokerURL}?token=$token");
                } catch (e) {
                  log("Error launching url ${widget.invokerURL}");
                }
              },
            ),
          ],
        );
      },
    );
  }

  @override
  void didUpdateWidget(Home oldWidget) {
    // this method IS called when parent widget passes new "props"
    // unlike React, this method IS called _before_ the build
    // unlike React, this method ISN'T called after setState()

    if (widget.host != oldWidget.host && widget.host != null) {
      setState(() {
        _host = widget.host;
      });
    }

    if (widget.mode != oldWidget.mode) {
      showMode(widget.mode);
    }

    super.didUpdateWidget(oldWidget);
  }

  showMode(mode) {
    if (mode == "app") {
      openVereign();
    } else if (mode == "oauth") {
      startOAuth();
    }
  }

  openVereign() {
    FlutterWebBrowser.openWebPage(url: _host, androidToolbarColor: Color(0xFFd51d32));
  }

  startOAuth() {
    var params = Config.getOAuthParams(host: _host);

    FlutterAppAuthWrapper.startAuth(
      AuthConfig(
        clientId: params["clientId"],
        clientSecret: params["clientSecret"],
        redirectUrl: params["redirectUrl"],
        state: "login",
        prompt: "consent",
        endpoint: AuthEndpoint(
            auth: params["authEndpoint"], token: params["tokenEndpoint"]),
        scopes: [
          "user_account_status",
          "user_territory",
          "user_profile"
        ],
      ),
    );
  }



  @override
  Widget build(BuildContext context) {
    var children = <Widget>[
      _urlButton(context, "Open Dashboard", openVereign),
      _urlButton(context, "Authorize with Vereign", startOAuth)
    ];

    if (Config.appFlavor == Flavor.DEVELOPMENT) {
      children.add(
        wrapInContainer(
          DropdownButton<String>(
            value: _host,
            onChanged: (String newValue) {
              setState(() {
                _host = newValue;
              });
            },
            items: hosts
                .map<DropdownMenuItem<String>>((String value) {
                  return DropdownMenuItem<String>(
                    value: value,
                    child: Text(value),
                  );
                })
                .toList(),
            )
        )
      );
    }

    return Scaffold(
        body: SafeArea(
            child: SingleChildScrollView(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: children
                ))));
  }

  Widget wrapInContainer(Widget widget) {
    return Container(
      padding: EdgeInsets.all(10.0),
      child: widget,
    );
  }

  Widget _urlButton(BuildContext context, String title, listener) {
    return wrapInContainer(
        FlatButton(
          color: Theme.of(context).primaryColor,
          textColor: Colors.white,
          padding: const EdgeInsets.symmetric(horizontal: 50.0, vertical: 15.0),
          child: Text(title),
          onPressed: listener
        )
    );
  }
}